module Filter where

import SimpleParser
import Gale
import Char
import GenUtil
import List

------------------------
-- Filter implementation
------------------------

data Filter = FilterSub String | FilterAuthor Category | FilterCategory Category  | 
    FilterKeyword String | FilterSender String | FilterNot Filter |
    FilterOr [Filter] | FilterAnd [Filter]
	    deriving(Eq)

parseFilter :: String -> Maybe Filter
parseFilter = parser (between spaces eof fp) where
    mp = many (token fp) >>= \v -> case v of
	    [x] -> return x
	    xs -> return $ FilterAnd xs
    fp = fmap FilterSub (token string)
    string = qstring <|> many (noneOf " \t\n~'")
    qstring = between (char '\'') (char '\'') $ many ((char '\'' >> char '\'' >> return '\'') <|> noneOf "'")


applyFilterStack :: [Filter] -> [(Int, Puff)] -> [(Int, Puff)]
applyFilterStack fs = concatMap f where
    f a@(_,p) | all (flip filterAp p) fs = [a]
    f _ = []

isPuff (Puff _ _) = True
isPuff _ = False

filterAp :: Filter -> Puff -> Bool
filterAp _ p | not (isPuff p) = True
filterAp (FilterNot f) p = not $ filterAp f p
filterAp (FilterSender s) p = case getFragmentString p "message/sender" of
    (Just s') | s' == s -> True
    _ -> False
filterAp (FilterKeyword k) p = any (k `isPrefixOf`) $ getFragmentStrings p "message.keyword"
filterAp (FilterCategory c) (Puff cs _) = any (`subCategory` c) cs
filterAp (FilterSub s) p = case getFragmentString p "message/body" of
    Just b | any (s `isPrefixOf`) (tails b) ->  True
    _ -> False
filterAp (FilterAnd fs) p = all (`filterAp` p) fs
filterAp (FilterOr fs) p = any (`filterAp` p) fs
filterAp _ _ = True

showFilter :: Filter -> String
showFilter (FilterSub s) = s
showFilter (FilterAuthor a) = "~a:" ++ catShowNew a
showFilter (FilterSender s) = "~s:" ++ s
showFilter (FilterCategory c) = "~c:" ++ catShowNew c
showFilter (FilterKeyword c) = "~k:" ++ c
showFilter (FilterNot f) = "!" ++ showFilter f
showFilter (FilterAnd fs) = "(" ++ concat (intersperse " & " (map showFilter fs)) ++ ")"
showFilter (FilterOr fs) = "(" ++ (concat $ intersperse " | " (map showFilter fs)) ++ ")"
--showFilter _ = ""
