-- $Id: SimpleParser.hs,v 1.2 2002/05/18 02:52:06 john Exp john $

-- Copyright (c) 2002 John Meacham (john@foo.net)
-- 
-- Permission is hereby granted, free of charge, to any person obtaining a
-- copy of this software and associated documentation files (the
-- "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish,
-- distribute, sublicense, and/or sell copies of the Software, and to
-- permit persons to whom the Software is furnished to do so, subject to
-- the following conditions:
-- 
-- The above copyright notice and this permission notice shall be included
-- in all copies or substantial portions of the Software.
-- 
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-- OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-- MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-- IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-- CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-- TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

module SimpleParser where



import Monad(liftM)
import Char
import Monad

infixr 1 <|>

-- very simple parser combinators with failure but limited non-determinism.
-- designed for parsing single lines or simple expressions.
-- advantages: pure haskell 98, simple.

newtype Parser a = MkP (String ->  Maybe (a,String))


app (MkP fn) s = fn s

instance Monad Parser where
    return a = MkP (\s -> (Just (a,s)))
    (MkP p) >>= q = MkP $ \s ->  (maybe Nothing (\(v,s') -> app (q v) s') (p s))
    fail _ = MkP $ \_ ->  Nothing

instance Functor Parser where
    fmap = liftM


instance MonadPlus Parser where
    mzero = MkP (\_ -> Nothing)
    mplus = (<|>) 


char :: Char -> Parser Char
char c = sat (== c)

eof :: Parser ()
eof = MkP eof' where
    eof' "" = Just ((),"")
    eof' _ = Nothing

oneOf cs = sat (`elem` cs)
noneOf cs = sat (`notElem` cs)

between o c p = do
    o
    v <- p
    c
    return v

option x p = p <|> return x
skipOption p = option () $ liftM (const ()) p
choice ps = foldl (<|>) mzero ps

anyChar = satisfy (const True)

upper = satisfy isUpper
lower = satisfy isLower
alphaNum = satisfy isAlphaNum
digit = satisfy isDigit
newline = char '\n'
space = satisfy isSpace
spaces = skipMany space


sat p = MkP f where 
    f (x:xs) | p x = Just (x,xs)  
    f _ = Nothing

satisfy = sat

p <|> q = MkP $ \s -> maybe (app q s) Just (app p s) 

many :: Parser a -> Parser [a]
many p = (p >>= \x -> many p >>= \xs -> return (x:xs)) <|> return []

many1 :: Parser a -> Parser [a]
many1 p = p >>= \x -> many p >>= \xs -> return (x:xs)

whitespace = skipMany (sat isSpace)

token p = p >>= \x -> whitespace >> return x
    
skipMany p = (p >> skipMany p) <|> return ()

number :: Parser Int
number = token (liftM read $ many1 (sat isDigit))

word :: Parser String
word = token $ (many1 (sat isAlpha))

exactWord s = do
    w <- word
    if w == s then return w else mzero


parser (MkP fn) s = case fn s of
    Just (v,"") -> Just v
    _ -> Nothing


{-
parse_file = do
    whitespace
    u <- number
    v <- number
    exactWord "done"
    return (u,v)

main = do
    c <- getContents
    let v =  parser parse_file c
    print v
    
-}
